#include "test_memory.h"


#include <drv_ioport.h>

extern ioport_t* ioport;

//..............................................................................
memory_kind_t memory_under_test = e_shared_sdram;
uint32_t walk_start[2] = {0xFFFFFFFE, 0x00000001};
//..............................................................................



//..............................................................................
uint32_t test_memory_rotate(uint32_t inval)
{
    uint32_t result = 0;
    uint32_t mask   = 0;

    result  = inval << 1;
    mask    = 0x80000000 & inval;
    mask    = (mask >> 31);
    mask   &= 0x00000001;
    result |= mask;

    return result;
}
//..............................................................................

//..............................................................................
bool test_memory_walk_data(uint32_t base, uint32_t size, test_memory_walk_kind_t walk_kind)
{
    bool result = true;
    uint32_t offset;
    uint32_t data;
    uint32_t retval;
    uint32_t i;
    int memsize = size/sizeof(uint32_t);
    uint8_t led = 0;

    offset = 0;
    data   = walk_start[walk_kind];

    for (i=0; i<memsize; i+=1)
    {
        *((uint32_t*)(base + offset)) = data;
        offset += 4;
        if (walk_kind == e_walk_data_one) data <<= 1;
        else                              data = test_memory_rotate(data); // RAF: declare macro instead of function call
    }

    offset = 0;
    data   = walk_start[walk_kind];
    for (i=0; i<memsize; i+=1)
    {

#if 0
        if (ioport!=0)
        {
                if (led > 7)
                {
                   led = 0;
                }

                ioport_set_value( ioport, 0, 1<<led );
                ioport_set_value( ioport, 1, 1<<led );
                led++;
        }
#endif        

        retval = *((uint32_t*)(base + offset));
        if (data != retval)
        {
            result = false;
            break;
        }
        offset += 4;
        if (walk_kind == e_walk_data_one) data <<= 1;
        else                              data = test_memory_rotate(data); // RAF: declare macro instead of function call
    }

    return result;
}
//..............................................................................

//..............................................................................
bool test_memory_walk_address_one(uint32_t base, uint32_t size)
{
    bool result = true;

    static const uint32_t start_address   = 0x00000004;
    static const uint32_t test_pattern    = 0x55555555;
    static const uint32_t test_pattern_cp = 0xAAAAAAAA;
    uint32_t mask;
    uint32_t retval;
    uint32_t retval2;
    uint32_t i;
    uint32_t j;

    mask = (size-1);
    // fill memory locations with a single 1 in address with arbitrary pattern
    for (i=start_address; (i&mask); i <<= 1)
        *(uint32_t*)(base+(i&mask)) = test_pattern;

    // fill the rest of memory with complement of test pattern
    for (i=start_address-4; (i&mask); i <<= 1)
        *(uint32_t*)(base+(i&mask)-4) = test_pattern_cp;

    // read out locations with a single 1 in address and check if arbitrary pattern is still there
    for (i=start_address; (i&mask); i <<= 1)
    {
        retval = *(uint32_t*)(base+(i&mask));
        // read value few times and check if it's stable
        for (j=0; j<10; j++)
        {
            retval2 = *(uint32_t*)(base+(i&mask));
            if (retval!=retval2)
            {
                result = false;
                break;
            }
        }
        if (result)
        {
            if (retval!=test_pattern)
            {
                result = false;
                break;
            }
        }
    }
    return result;
}
//..............................................................................

//..............................................................................
bool test_memory_walk_address_zero(uint32_t base, uint32_t size)
{
    bool result = true;
    static const uint32_t start_address   = 0xFFFFFFFB;
    static const uint32_t test_pattern    = 0x55555555;
    static const uint32_t test_pattern_cp = 0xAAAAAAAA;
    static const uint32_t address_mask    = 0xFFFFFFFC;
    uint32_t mask;
    uint32_t retval;
    uint32_t retval2;
    uint32_t i;
    uint32_t j;

    mask = (size-1);
    // fill memory locations with a single 1 in address with arbitrary pattern
    for (i=start_address; (i&mask)!=mask; i = test_memory_rotate(i))
        *(uint32_t*)((base+(i&mask))& address_mask) = test_pattern;

    // fill the rest of memory with complement of test pattern
    for (i=start_address+4; (i&mask)!=mask; i = test_memory_rotate(i))
        *(uint32_t*)((base+(i&mask)+4)& address_mask) = test_pattern_cp;

    // read out locations with a single 1 in address and check if arbitrary pattern is still there
    for (i=start_address; (i&mask)!=mask;  i = test_memory_rotate(i))
    {
        retval = *(uint32_t*)((base+(i&mask))&address_mask);
        for (j=0; j<10; j++)
        {
            retval2 = *(uint32_t*)((base+(i&mask))&address_mask);
            if (retval!=retval2)
            {
                result = false;
                break;
            }
        }
        if (result)
        {
            if (retval!=test_pattern)
            {
                result = false;
                break;
            }
        }
    }

    return result;
}
//..............................................................................

//..............................................................................
bool test_memory_byte_enable(uint32_t base, uint32_t size)
{
    bool result = true;
    uint32_t i;
    uint8_t  retval;

    for (i=0; i<32; i+=1)
        *((uint8_t*)(base + i)) = i;

    for (i=0; i<32; i+=1)
    {
        retval = *((uint8_t*)(base + i));
        if (retval != (uint8_t) i)
        {
            result = false;
            break;
        }
    }

    return result;
}
//..............................................................................

